@@ -1,21 +1,22 @@
-/* Capstone Disassembler Engine */
+/* Capstone Disassembly Engine */
 /* By Dang Hoang Vu <danghvu@gmail.com> 2013 */
 
+#ifdef CAPSTONE_HAS_MIPS
+
 #include "../../utils.h"
 #include "../../MCRegisterInfo.h"
 #include "MipsDisassembler.h"
 #include "MipsInstPrinter.h"
 #include "MipsMapping.h"
 
-void enable_mips() {};
-
 static cs_err init(cs_struct *ud)
 {
 	MCRegisterInfo *mri;
 
 	// verify if requested mode is valid
 	if (ud->mode & ~(CS_MODE_LITTLE_ENDIAN | CS_MODE_32 | CS_MODE_64 |
-				CS_MODE_MICRO | CS_MODE_N64 | CS_MODE_BIG_ENDIAN))
+				CS_MODE_MICRO | CS_MODE_MIPS32R6 |
+				CS_MODE_MIPSGP64 | CS_MODE_BIG_ENDIAN))
 		return CS_ERR_MODE;
 
 	mri = cs_mem_malloc(sizeof(*mri));
@@ -27,8 +28,9 @@ static cs_err init(cs_struct *ud)
 	ud->reg_name = Mips_reg_name;
 	ud->insn_id = Mips_get_insn_id;
 	ud->insn_name = Mips_insn_name;
+	ud->group_name = Mips_group_name;
 
-	if (ud->mode & CS_MODE_32)
+	if (ud->mode & CS_MODE_32 || ud->mode & CS_MODE_MIPS32R6)
 		ud->disasm = Mips_getInstruction;
 	else
 		ud->disasm = Mips64_getInstruction;
@@ -62,3 +64,5 @@ void Mips_enable(void)
 	// support this arch
 	all_arch |= (1 << CS_ARCH_MIPS);
 }
+
+#endif
