@@ -13,29 +13,16 @@
  *
  *===----------------------------------------------------------------------===*/
 
-/* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013> */
+/* Capstone Disassembly Engine */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
 
 #ifndef CS_X86_DISASSEMBLERDECODER_H
 #define CS_X86_DISASSEMBLERDECODER_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-#define INSTRUCTION_SPECIFIER_FIELDS \
-  uint16_t operands;
-
-#define INSTRUCTION_IDS     \
-  uint16_t instructionIDs;
-
 #include <stdio.h>
 
 #include "X86DisassemblerDecoderCommon.h"
 
-#undef INSTRUCTION_SPECIFIER_FIELDS
-#undef INSTRUCTION_IDS
-
 #include <stdint.h>
 
 /*
@@ -52,6 +39,21 @@ extern "C" {
 #define xFromREX(rex)        (((rex) & 0x2) >> 1)
 #define bFromREX(rex)        ((rex) & 0x1)
 
+#define rFromEVEX2of4(evex)     (((~(evex)) & 0x80) >> 7)
+#define xFromEVEX2of4(evex)     (((~(evex)) & 0x40) >> 6)
+#define bFromEVEX2of4(evex)     (((~(evex)) & 0x20) >> 5)
+#define r2FromEVEX2of4(evex)    (((~(evex)) & 0x10) >> 4)
+#define mmFromEVEX2of4(evex)    ((evex) & 0x3)
+#define wFromEVEX3of4(evex)     (((evex) & 0x80) >> 7)
+#define vvvvFromEVEX3of4(evex)  (((~(evex)) & 0x78) >> 3)
+#define ppFromEVEX3of4(evex)    ((evex) & 0x3)
+#define zFromEVEX4of4(evex)     (((evex) & 0x80) >> 7)
+#define l2FromEVEX4of4(evex)    (((evex) & 0x40) >> 6)
+#define lFromEVEX4of4(evex)     (((evex) & 0x20) >> 5)
+#define bFromEVEX4of4(evex)     (((evex) & 0x10) >> 4)
+#define v2FromEVEX4of4(evex)    (((~evex) & 0x8) >> 3)
+#define aaaFromEVEX4of4(evex)   ((evex) & 0x7)
+
 #define rFromVEX2of3(vex)       (((~(vex)) & 0x80) >> 7)
 #define xFromVEX2of3(vex)       (((~(vex)) & 0x40) >> 6)
 #define bFromVEX2of3(vex)       (((~(vex)) & 0x20) >> 5)
@@ -322,6 +324,16 @@ extern "C" {
   ENTRY(ZMM30)    \
   ENTRY(ZMM31)
 
+#define REGS_MASKS \
+  ENTRY(K0)        \
+  ENTRY(K1)        \
+  ENTRY(K2)        \
+  ENTRY(K3)        \
+  ENTRY(K4)        \
+  ENTRY(K5)        \
+  ENTRY(K6)        \
+  ENTRY(K7)
+
 #define REGS_SEGMENT \
   ENTRY(ES)          \
   ENTRY(CS)          \
@@ -369,6 +381,7 @@ extern "C" {
   REGS_XMM            \
   REGS_YMM            \
   REGS_ZMM            \
+  REGS_MASKS          \
   REGS_SEGMENT        \
   REGS_DEBUG          \
   REGS_CONTROL        \
@@ -457,7 +470,6 @@ typedef enum {
 /*
  * VEXLeadingOpcodeByte - Possible values for the VEX.m-mmmm field
  */
-
 typedef enum {
   VEX_LOB_0F = 0x1,
   VEX_LOB_0F38 = 0x2,
@@ -471,9 +483,8 @@ typedef enum {
 } XOPMapSelect;
 
 /*
- * VEXPrefixCode - Possible values for the VEX.pp field
+ * VEXPrefixCode - Possible values for the VEX.pp/EVEX.pp field
  */
-
 typedef enum {
   VEX_PREFIX_NONE = 0x0,
   VEX_PREFIX_66 = 0x1,
@@ -482,13 +493,18 @@ typedef enum {
 } VEXPrefixCode;
 
 typedef enum {
-  TYPE_NO_VEX_XOP = 0x0,
-  TYPE_VEX_2B = 0x1,
-  TYPE_VEX_3B = 0x2,
-  TYPE_XOP = 0x3
-} VEXXOPType;
+	TYPE_NO_VEX_XOP   = 0x0,
+	TYPE_VEX_2B       = 0x1,
+	TYPE_VEX_3B       = 0x2,
+	TYPE_EVEX         = 0x3,
+	TYPE_XOP          = 0x4
+} VectorExtensionType;
 
-typedef uint8_t BOOL;
+struct reader_info {
+	const uint8_t *code;
+	uint64_t size;
+	uint64_t offset;
+};
 
 /*
  * byteReader_t - Type for the byte reader that the consumer must provide to
@@ -501,7 +517,7 @@ typedef uint8_t BOOL;
  *                  be read from.
  * @return        - -1 if the byte cannot be read for any reason; 0 otherwise.
  */
-typedef int (*byteReader_t)(const void* arg, uint8_t* byte, uint64_t address);
+typedef int (*byteReader_t)(const struct reader_info *arg, uint8_t* byte, uint64_t address);
 
 /*
  * dlog_t - Type for the logging function that the consumer can provide to
@@ -513,11 +529,53 @@ typedef int (*byteReader_t)(const void* arg, uint8_t* byte, uint64_t address);
  */
 typedef void (*dlog_t)(void* arg, const char *log);
 
+/// The specification for how to extract and interpret a full instruction and
+/// its operands.
+struct InstructionSpecifier {
+	uint16_t operands;
+};
+
 /*
  * The x86 internal instruction, which is produced by the decoder.
  */
 typedef struct InternalInstruction {
+  // from here, all members must be initialized to ZERO to work properly
   uint8_t operandSize;
+  /* 1 if the prefix byte corresponding to the entry is present; 0 if not */
+  uint8_t prefixPresent[0x100];
+  uint8_t prefix0, prefix1, prefix2, prefix3;
+  /* The value of the REX prefix, if present */
+  uint8_t rexPrefix;
+  /* The segment override type */
+  SegmentOverride segmentOverride;
+  bool                          consumedModRM;
+  uint8_t                       orgModRM;  // save original modRM because we will modify modRM
+  /* The SIB byte, used for more complex 32- or 64-bit memory operands */
+  bool                          consumedSIB;
+  uint8_t                       sib;
+  /* The displacement, used for memory operands */
+  bool                          consumedDisplacement;
+  int32_t                       displacement;
+  /* The value of the two-byte escape prefix (usually 0x0f) */
+  uint8_t twoByteEscape;
+  /* The value of the three-byte escape prefix (usually 0x38 or 0x3a) */
+  uint8_t threeByteEscape;
+  /* SIB state */
+  SIBIndex                      sibIndex;
+  uint8_t                       sibScale;
+  SIBBase                       sibBase;
+  uint8_t                       numImmediatesConsumed;
+  /* 1 if the prefix byte, 0xf2 or 0xf3 is xacquire or xrelease */
+  bool xAcquireRelease;
+
+  /* contains the location (for use with the reader) of the prefix byte */
+  uint64_t prefixLocations[0x100];
+
+  /* The value of the vector extension prefix(EVEX/VEX/XOP), if present */
+  uint8_t vectorExtensionPrefix[4];
+
+  // end-of-zero-members
+
   /* Reader interface (C) */
   byteReader_t reader;
   /* Opaque value passed to the reader */
@@ -541,23 +599,11 @@ typedef struct InternalInstruction {
 
   /* Prefix state */
 
-  /* 1 if the prefix byte corresponding to the entry is present; 0 if not */
-  uint8_t prefixPresent[0x100];
-  /* contains the location (for use with the reader) of the prefix byte */
-  uint64_t prefixLocations[0x100];
-  /* The value of the VEX/XOP prefix, if present */
-  uint8_t vexXopPrefix[3];
-  /* The length of the VEX prefix (0 if not present) */
-  VEXXOPType vexXopType;
-  /* The value of the REX prefix, if present */
-  uint8_t rexPrefix;
+  /* The type of the vector extension prefix */
+  VectorExtensionType vectorExtensionType;
   /* The location where a mandatory prefix would have to be (i.e., right before
      the opcode, or right before the REX prefix if one is present) */
   uint64_t necessaryPrefixLocation;
-  /* The segment override type */
-  SegmentOverride segmentOverride;
-  /* 1 if the prefix byte, 0xf2 or 0xf3 is xacquire or xrelease */
-  BOOL xAcquireRelease;
 
   /* Sizes of various critical pieces of data, in bytes */
   uint8_t registerSize;
@@ -565,6 +611,8 @@ typedef struct InternalInstruction {
   uint8_t displacementSize;
   uint8_t immediateSize;
 
+  uint8_t immSize;	// immediate size for X86_OP_IMM operand
+
   /* Offsets from the start of the instruction to the pieces of data, which is
      needed to find relocation entries for adding symbolic operands */
   uint8_t displacementOffset;
@@ -572,10 +620,6 @@ typedef struct InternalInstruction {
 
   /* opcode state */
 
-  /* The value of the two-byte escape prefix (usually 0x0f) */
-  uint8_t twoByteEscape;
-  /* The value of the three-byte escape prefix (usually 0x38 or 0x3a) */
-  uint8_t threeByteEscape;
   /* The last byte of the opcode, not counting any ModR/M extension */
   uint8_t opcode;
 
@@ -596,27 +640,21 @@ typedef struct InternalInstruction {
      instructions */
   Reg                           vvvv;
 
+  /* The writemask for AVX-512 instructions which is contained in EVEX.aaa */
+  Reg                           writemask;
+
   /* The ModR/M byte, which contains most register operands and some portion of
      all memory operands */
-  BOOL                          consumedModRM;
   uint8_t                       modRM;
 
-  /* The SIB byte, used for more complex 32- or 64-bit memory operands */
-  BOOL                          consumedSIB;
-  uint8_t                       sib;
-
-  /* The displacement, used for memory operands */
-  BOOL                          consumedDisplacement;
-  int32_t                       displacement;
+  // special data to handle MOVcr, MOVdr, MOVrc, MOVrd
+  uint8_t                       firstByte;     // save the first byte in stream
 
   /* Immediates.  There can be two in some cases */
-  uint8_t                       numImmediatesConsumed;
   uint8_t                       numImmediatesTranslated;
   uint64_t                      immediates[2];
 
   /* A register or immediate operand encoded into the opcode */
-  BOOL                          consumedOpcodeModifier;
-  uint8_t                       opcodeModifier;
   Reg                           opcodeRegister;
 
   /* Portions of the ModR/M byte */
@@ -634,11 +672,6 @@ typedef struct InternalInstruction {
   /* The reg field always encodes a register */
   Reg                           reg;
 
-  /* SIB state */
-  SIBIndex                      sibIndex;
-  uint8_t                       sibScale;
-  SIBBase                       sibBase;
-
   const struct OperandSpecifier *operands;
 } InternalInstruction;
 
@@ -666,8 +699,4 @@ int decodeInstruction(struct InternalInstruction* insn,
 
 //const char *x86DisassemblerGetInstrName(unsigned Opcode, const void *mii);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
